/*Class name: FileIO
 *Namespace: DVM_Management_Center.FileIO
 *Author: Harald Gsoedl
 *Last change: Benno Rott
 *Changed: 21/02/2007
 *Use: Provides routines for basic file I/O-Functions
 */

using System;
using System.IO;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;

namespace DVM_Management_Center.FileIO
{
    /// <summary>
    /// Provides routines for the basic I/O-Functions
    /// </summary>
    class FileIO
    {
        const string CURRENT_LOG_FILE = "/current.log";
        const string OLD_LOG_FILE = "/old.log";
        const string START_FILE = "/start.conf";

        /// <summary>
        /// Methode to save the current running-status
        /// </summary>
        /// <param name="status">Boolean: 1 or 0</param>
        /// <param name="mapFile">The current map-file. Relative address to the Application-Startup-Path</param>
        public void WriteRunStatus(bool status, String mapFile)
        {
            try
            {
                String[] conf = File.ReadAllLines(Application.StartupPath + START_FILE);
                if (conf.Length > 1)
                {
                    if (status)
                        conf[1] = "1";
                    else
                        conf[1] = "0";
                }
                else if (conf.Length == 1)
                {
                    String[] temp = new String[2];
                    temp[0] = conf[0];
                    temp[1] = (status) ? "1" : "0";
                    conf = temp;
                }
                else
                {
                    String[] temp = new String[2];
                    temp[0] = mapFile;
                    temp[1] = (status) ? "1" : "0";
                    conf = temp;
                }
                File.WriteAllLines(Application.StartupPath + START_FILE, conf);
            }
            catch
            {
                System.Windows.Forms.MessageBox.Show("The current running status could not be saved.");
            }
        }

        /// <summary>
        /// Methode to add a line in the log-file
        /// </summary>
        /// <param name="entry">The newest log-entry</param>
        public void WriteLogFile(String entry)
        {
            try
            {
                try
                {
                    FileInfo fileInfo = new FileInfo(System.Windows.Forms.Application.StartupPath + CURRENT_LOG_FILE);
                    if (fileInfo.Length > 1048576) // File-size bigger than 1 MB
                    {
                        File.Delete(System.Windows.Forms.Application.StartupPath + OLD_LOG_FILE);
                        fileInfo.MoveTo(System.Windows.Forms.Application.StartupPath + OLD_LOG_FILE);
                    }
                }
                finally
                {
                    StreamWriter sw = new StreamWriter(System.Windows.Forms.Application.StartupPath + CURRENT_LOG_FILE, true);
                    sw.WriteLine(entry);
                    sw.Close();
                }
            }
            catch (Exception ex)
            {
                throw new Exception("There was an error while writing the log-file. No logs written.\n" + ex.Message);
            }
        }

        /// <summary>
        /// Method to read the log file
        /// </summary>
        /// <returns>String[] with the data-lines</returns>
        public String[] ReadLogFile()
        {
            String[] logs = new String[0];
            try
            {
                logs = File.ReadAllLines(System.Windows.Forms.Application.StartupPath + CURRENT_LOG_FILE);
                Array.Reverse(logs);
            }
            catch (FileNotFoundException)
            {
                throw new Exception("The log file was not found.");
            }
            catch (Exception ex)
            {
                throw new Exception("The log file could not be opened: " + ex.Message);
            }
            return logs;
        }

        /// <summary>
        /// Methode to read the last log-entry
        /// </summary>
        /// <returns></returns>
        public string ReadLastLog()
        {
            String result = String.Empty;
            try
            {
                String[] lines = File.ReadAllLines(System.Windows.Forms.Application.StartupPath + CURRENT_LOG_FILE);
                if (lines.Length == 0)
                    result = "0";
                else
                    result = lines[lines.Length - 1];
            }
            catch (Exception)
            {
                MessageBox.Show("The last log entry could not be read.");
                result = "0";
            }
            return result;
        }

        /// <summary>
        /// Method to clear a file
        /// </summary>
        /// <param name="path">path is the file to be cleared</param>
        public void ClearFile(String path)
        {
            File.WriteAllText(path, "");
        }
    }
}