using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Threading;
using System.IO;
using System.Xml;

using DVM_Management_Center.SNMP;
using DVM_Management_Center.FileIO;
using DVM_Management_Center.Mail;
using DVM_Management_Center.Icons;
using DVM_Management_Center.Objects;

namespace DVM_Management_Center.GUI
{
    public partial class Main : Form
    {
        System.Diagnostics.Process proc;
        Hashtable leds, resStorage, mapTexts;
        Thread[] dvmThreads;
        MAPObject mapObject;
        SendAlert sendAlert;
        String mapFile;
        Boolean run;

        const String START_FILE = "/start.conf";
        const String MAIL_CONFIG_FILE = "/mailsettings.xml";
        const String CURRENT_LOG_FILE = "/current.log";

        public Main()
        {
            InitializeComponent();
        }

        private void LbDVMClick(object sender, EventArgs e)
        {
            CloseProcess("DVM MPEG");
            String ip;
            if (((Label)sender).Name.Substring(0, 3) == "lbD")
                ip = mapObject.DVMObjects[Int32.Parse(((Label)sender).Name.Substring(5, ((Label)sender).Name.Length - 5))].Ip;
            else
                ip = mapObject.DVMObjects[Int32.Parse(((Label)sender).Name.Substring(9, ((Label)sender).Name.Length - 9))].Ip;
            proc.StartInfo.Arguments = ip + @"/dvm/dvm.jnlp";
            proc.StartInfo.FileName = "IExplore.exe";
            proc.Start();
        }

        private void lbDVMMouseOver(object sender, EventArgs e)
        {
            int index;
            if (((Label)sender).Name.Substring(0, 3) == "lbD") // it's a lbDVMx-Label
                index = Int32.Parse(((Label)sender).Name.Substring(5, ((Label)sender).Name.Length - 5));
            else // it's a lbMapx-Label
                index = Int32.Parse(((Label)sender).Name.Substring(9, ((Label)sender).Name.Length - 9));
            if (mapObject.DVMObjects.Length != 0)
            {
                mapObject.DVMObjects[index].Led.BringToFront();
                mapObject.DVMObjects[index].MapText.BringToFront();
            }
            else
                tbPviewMap.Controls.Remove((Label)sender);
        }

        /// <summary>
        /// The method initializes some variables and reads the maps from the CONFIG_FILE
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MainLoad(object sender, EventArgs e)
        {
            Thread.CurrentThread.Priority = ThreadPriority.AboveNormal;
            resStorage = new Hashtable();
            leds = new Hashtable();
            mapTexts = new Hashtable();
            proc = new System.Diagnostics.Process();
            run = false;
            try
            {
                String[] conf = File.ReadAllLines(Application.StartupPath + START_FILE);
                if (conf.Length != 0 && conf[0] != "")
                {
                    mapFile = conf[0];
                    this.LoadMapFromFile(Application.StartupPath + "/" + mapFile);

                    if (conf.Length > 1)
                    {
                        if (conf[1] == "1")
                            this.StartMonitoring();
                        else
                            this.StopMonitoring();
                    }
                    else
                        this.StopMonitoring();
                }
                else
                {
                    this.StopMonitoring();
                    Xml_Serializer xmlS = new Xml_Serializer(System.Windows.Forms.Application.StartupPath + "/mailsettings.xml");
                    sendAlert = new SendAlert(0, (AlertSettings)xmlS.DeserializeObject(typeof(AlertSettings)));
                }
            }
            catch (Exception ex)
            {
                Xml_Serializer xmlS = new Xml_Serializer(System.Windows.Forms.Application.StartupPath + "/mailsettings.xml");
                sendAlert = new SendAlert(0, (AlertSettings)xmlS.DeserializeObject(typeof(AlertSettings)));
                MessageBox.Show("The config file " + START_FILE + " could not be loaded!\n" + ex.Message);
            }
          
            try
            {
                FileIO.FileIO fileIO = new FileIO.FileIO();
                String[] logs = fileIO.ReadLogFile();
                lsVlogs.BeginUpdate();
                lsVlogs.Items.Clear();
                foreach (String log in logs)
                    lsVlogs.Items.Add(log);
                lsVlogs.EndUpdate();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
            SetAlertInfo();
            timer_lastlog.Start();
            gbErrorList.Text = "Log entries:\r\nListed entries in file " + Application.StartupPath + "\\current.log:";
        }

        private void exitToolStripMenuItemClick(object sender, EventArgs e)
        {
            this.Close();
        }

        /// <summary>
        /// This method initialize the DVM-Objects of the current Map and paints the labels on the Picture
        /// </summary>
        private void InitDvmObj()
        {
            // Initialize the threads
            dvmThreads = new Thread[mapObject.DVMObjects.Length];
            treeView.SuspendLayout();
            treeView.BeginUpdate();
            this.SuspendLayout();
            treeView.Nodes.Clear();

            for (int i = 0; i < mapObject.DVMObjects.Length; i++)
            {
                System.Windows.Forms.Label label;
                label = new System.Windows.Forms.Label();
                label.AutoSize = false;
                label.BackColor = System.Drawing.Color.Transparent;
                label.BackColor = System.Drawing.Color.DodgerBlue;
                label.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
                label.Cursor = System.Windows.Forms.Cursors.Hand;
                label.Font = new System.Drawing.Font("Microsoft Sans Serif", 7, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((byte)(0)));
                label.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
                label.Top = Convert.ToInt32(mapObject.DVMObjects[i].Top) + pBMap.Top;
                label.Left = Convert.ToInt32(mapObject.DVMObjects[i].Left) + pBMap.Left;
                label.Name = "lbDVM" + i.ToString();
                label.Size = new System.Drawing.Size(24, 16);
                label.Padding = new Padding(0);
                label.TabIndex = 1;
                label.Text = "";
                this.tbPviewMap.Controls.Add(label);
                label.Click += new System.EventHandler(this.LbDVMClick);
                label.MouseHover += new System.EventHandler(this.lbDVMMouseOver);
                label.BringToFront();

                System.Windows.Forms.Label mapText;
                mapText = new System.Windows.Forms.Label();
                mapText.AutoSize = false;
                mapText.BackColor = System.Drawing.Color.DodgerBlue;
                mapText.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
                mapText.Cursor = System.Windows.Forms.Cursors.Hand;
                mapText.Font = new System.Drawing.Font("Microsoft Sans Serif", 7, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((byte)(0)));
                mapText.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
                mapText.Top = Convert.ToInt32(mapObject.DVMObjects[i].Top) + 17 + pBMap.Top;
                mapText.Left = Convert.ToInt32(mapObject.DVMObjects[i].Left) - 63 + pBMap.Left;
                mapText.Name = "lbMapText" + i.ToString();
                mapText.Size = new System.Drawing.Size(150, 16);
                mapText.TabIndex = 1;
                mapText.Text = mapObject.DVMObjects[i].SiteName;
                this.tbPviewMap.Controls.Add(mapText);
                mapText.MouseHover += new System.EventHandler(this.lbDVMMouseOver);
                mapText.Click += new System.EventHandler(this.LbDVMClick);
                mapText.BringToFront();

                mapObject.DVMObjects[i].Led = label;
                mapObject.DVMObjects[i].MapText = mapText;
                mapObject.DVMObjects[i].InitNode();
                treeView.Nodes.Add(mapObject.DVMObjects[i].DvmNode);
                dvmThreads[i] = new Thread(new ThreadStart(mapObject.DVMObjects[i].SetDVM));
            }
            this.ResumeLayout();
            treeView.EndUpdate();
            treeView.ResumeLayout();
        }

        private void timer_checkTick(object sender, EventArgs e)
        {
            tlStrpStPr.Value = 0;
            timer_progress.Enabled = true;

             bool update = false;
             if (mapObject != null)

             for (int i = 0; i < mapObject.DVMObjects.Length; i++)
             {
                 if (!dvmThreads[i].IsAlive)
                 {
                     int conCnt = 0;
                     int syncCnt = 0;
                     int redCnt = 0;
                     int yelCnt = 0;
                     int greCnt = 0;

                     if (mapObject.DVMObjects[i].Ready == true)
                     {
                         if (update == false)
                         {
                             treeView.BeginUpdate();
                             update = true;
                         }
                         leds[i] = mapObject.DVMObjects[i].Led;
                         mapTexts[i] = mapObject.DVMObjects[i].MapText;

                         // Check the main point of the dvm
                         if (treeView.Nodes[i].ImageIndex != mapObject.DVMObjects[i].DvmNode.ImageIndex)
                         {
                             treeView.Nodes[i].ImageIndex = treeView.Nodes[i].SelectedImageIndex = mapObject.DVMObjects[i].DvmNode.ImageIndex;
                         }

                         if (mapObject.DVMObjects[i].DvmNode.ImageIndex == IconImagelist.MAP_CONNECTIVITY_LOSS)
                         { // dvm is offline
                             conCnt++;
                             treeView.Nodes[i].Remove();
                             treeView.Nodes.Insert(i, mapObject.DVMObjects[i].DvmNode);
                             ((Label)leds[i]).BackColor = Color.Gray;
                             ((Label)leds[i]).Text = "X";
                             treeView.Nodes[i].NodeFont = new System.Drawing.Font("Microsoft Sans Serif", 12, System.Drawing.FontStyle.Italic);
                         }
                         else // dvm is online
                         {
                             // Check the main text of the dvm
                             if (treeView.Nodes[i].Text != mapObject.DVMObjects[i].DvmNode.Text)
                             {
                                 treeView.Nodes[i].Text = mapObject.DVMObjects[i].DvmNode.Text;
                             }
                             if (treeView.Nodes[i].GetNodeCount(false) != mapObject.DVMObjects[i].DvmNode.GetNodeCount(false))
                             {
                                 bool expanded = treeView.Nodes[i].IsExpanded;
                                 treeView.Nodes[i].Nodes.Clear();
                                 for (int j = 0; j < mapObject.DVMObjects[i].DvmNode.GetNodeCount(false); j++)
                                 {
                                     treeView.Nodes[i].Nodes.Insert(j, mapObject.DVMObjects[i].DvmNode.Nodes[j].Text, mapObject.DVMObjects[i].DvmNode.Nodes[j].Text, mapObject.DVMObjects[i].DvmNode.Nodes[j].ImageIndex, mapObject.DVMObjects[i].DvmNode.Nodes[j].SelectedImageIndex);
                                     treeView.Nodes[i].Nodes[j].Tag = mapObject.DVMObjects[i].DvmNode.Nodes[j].Tag;
                                     if (treeView.Nodes[i].Nodes[j].GetNodeCount(false) != mapObject.DVMObjects[i].DvmNode.Nodes[j].GetNodeCount(false))
                                     {
                                         bool folder_expanded = treeView.Nodes[i].Nodes[j].IsExpanded;
                                         treeView.Nodes[i].Nodes[j].Nodes.Clear();
                                         for (int k = 0; k < mapObject.DVMObjects[i].DvmNode.Nodes[j].GetNodeCount(false); k++)
                                         {
                                             treeView.Nodes[i].Nodes[j].Nodes.Insert(k, mapObject.DVMObjects[i].DvmNode.Nodes[j].Nodes[k].Text, mapObject.DVMObjects[i].DvmNode.Nodes[j].Nodes[k].Text, mapObject.DVMObjects[i].DvmNode.Nodes[j].Nodes[k].ImageIndex, mapObject.DVMObjects[i].DvmNode.Nodes[j].Nodes[k].SelectedImageIndex);
                                         }
                                         if (folder_expanded)
                                             treeView.Nodes[i].Nodes[j].Expand();
                                     }
                                 }
                                 if (expanded)
                                     treeView.Nodes[i].Expand();
                             }
                             else
                             {
                                 // Check the inputs without folders and folders

                                 for (int j = 0; j < mapObject.DVMObjects[i].DvmNode.Nodes.Count; j++)
                                 {
                                     //Folder Icon
                                     if (!treeView.Nodes[i].Nodes[j].IsExpanded && treeView.Nodes[i].Nodes[j].ImageIndex != mapObject.DVMObjects[i].DvmNode.Nodes[j].ImageIndex)
                                     {
                                         treeView.Nodes[i].Nodes[j].ImageIndex = treeView.Nodes[i].Nodes[j].SelectedImageIndex = mapObject.DVMObjects[i].DvmNode.Nodes[j].ImageIndex;
                                         treeView.Nodes[i].Nodes[j].Tag = mapObject.DVMObjects[i].DvmNode.Nodes[j].ImageIndex;
                                     }
                                     else if (treeView.Nodes[i].Nodes[j].IsExpanded && treeView.Nodes[i].Nodes[j].Tag.ToString() != mapObject.DVMObjects[i].DvmNode.Nodes[j].ImageIndex.ToString())
                                     {
                                         treeView.Nodes[i].Nodes[j].Tag = mapObject.DVMObjects[i].DvmNode.Nodes[j].ImageIndex;
                                         treeView.Nodes[i].Nodes[j].ImageIndex = IconImagelist.OPENED_FOLDER;
                                     }

                                     if (treeView.Nodes[i].Nodes[j].Text != mapObject.DVMObjects[i].DvmNode.Nodes[j].Text)
                                         treeView.Nodes[i].Nodes[j].Text = mapObject.DVMObjects[i].DvmNode.Nodes[j].Text;

                                     if (treeView.Nodes[i].Nodes[j].GetNodeCount(false) != mapObject.DVMObjects[i].DvmNode.Nodes[j].GetNodeCount(false))
                                     {
                                         bool folder_expanded = treeView.Nodes[i].Nodes[j].IsExpanded;
                                         treeView.Nodes[i].Nodes[j].Nodes.Clear();
                                         for (int k = 0; k < mapObject.DVMObjects[i].DvmNode.Nodes[j].GetNodeCount(false); k++)
                                         {
                                             treeView.Nodes[i].Nodes[j].Nodes.Insert(k, mapObject.DVMObjects[i].DvmNode.Nodes[j].Nodes[k].Text, mapObject.DVMObjects[i].DvmNode.Nodes[j].Nodes[k].Text, mapObject.DVMObjects[i].DvmNode.Nodes[j].Nodes[k].ImageIndex, mapObject.DVMObjects[i].DvmNode.Nodes[j].Nodes[k].SelectedImageIndex);
                                         }
                                         if (folder_expanded)
                                             treeView.Nodes[i].Nodes[j].Expand();
                                     }
                                     else
                                     {
                                         for (int k = 0; k < mapObject.DVMObjects[i].DvmNode.Nodes[j].GetNodeCount(false); k++)
                                         {
                                             if (treeView.Nodes[i].Nodes[j].Nodes[k].ImageIndex != mapObject.DVMObjects[i].DvmNode.Nodes[j].Nodes[k].ImageIndex)
                                             {
                                                 treeView.Nodes[i].Nodes[j].Nodes[k].ImageIndex = treeView.Nodes[i].Nodes[j].Nodes[k].SelectedImageIndex = mapObject.DVMObjects[i].DvmNode.Nodes[j].Nodes[k].ImageIndex;
                                             }
                                             if (treeView.Nodes[i].Nodes[j].Nodes[k].Tag != mapObject.DVMObjects[i].DvmNode.Nodes[j].Nodes[k].Tag)
                                             {
                                                 treeView.Nodes[i].Nodes[j].Nodes[k].Tag = treeView.Nodes[i].Nodes[j].Nodes[k].Tag = mapObject.DVMObjects[i].DvmNode.Nodes[j].Nodes[k].Tag;
                                             }
                                             if (treeView.Nodes[i].Nodes[j].Nodes[k].Text != mapObject.DVMObjects[i].DvmNode.Nodes[j].Nodes[k].Text)
                                             {
                                                 treeView.Nodes[i].Nodes[j].Nodes[k].Text = mapObject.DVMObjects[i].DvmNode.Nodes[j].Nodes[k].Text;
                                             }
                                         }
                                     }
                                 }
                             }
                              
                             if (mapObject.DVMObjects[i].DvmNode.Nodes.Count > 0)
                             {
                                 ((Label)mapTexts[i]).Text = mapObject.DVMObjects[i].SiteName;
                                 if (treeView.Nodes[i].ImageIndex == IconImagelist.MAP_TS_SYNC)
                                 {
                                     ((Label)leds[i]).BackColor = Color.Red;
                                     ((Label)leds[i]).Text = "";
                                     syncCnt++;
                                 }
                                 else if (treeView.Nodes[i].ImageIndex == IconImagelist.MAP_CURRENT_ALARM)
                                 {
                                     ((Label)leds[i]).BackColor = Color.Red;
                                     ((Label)leds[i]).Text = "";
                                     redCnt++;
                                 }
                                 else if (treeView.Nodes[i].ImageIndex == IconImagelist.MAP_HISTORY_ALARM)
                                 {
                                     ((Label)leds[i]).BackColor = Color.Yellow;
                                     ((Label)leds[i]).Text = "";
                                     yelCnt++;
                                 }
                                 else if (treeView.Nodes[i].ImageIndex == IconImagelist.MAP_NO_MONITORING)
                                 {
                                     ((Label)leds[i]).BackColor = Color.LightGray;
                                     ((Label)leds[i]).Text = "";
                                 }
                                 else
                                 {
                                     ((Label)leds[i]).BackColor = Color.Lime;
                                     ((Label)leds[i]).Text = "";
                                     greCnt++;
                                 }
                             }
                             else // no node, no port
                             {
                                 treeView.Nodes[i].Remove();
                                 treeView.Nodes.Insert(i, mapObject.DVMObjects[i].DvmNode);
                                 ((Label)leds[i]).BackColor = Color.Gray;
                                 ((Label)leds[i]).Text = "";
                             }
                             ((Label)mapTexts[i]).Text = mapObject.DVMObjects[i].SiteName.Replace("&", "&&");
                             treeView.Nodes[i].NodeFont = new System.Drawing.Font("Microsoft Sans Serif", 12, System.Drawing.FontStyle.Bold);
                         }
                         for (int j = 0; j < treeView.Nodes[i].Nodes.Count; j++)
                         {
                             if (treeView.Nodes[i].Nodes[j].GetNodeCount(false) == 0)
                             {
                                treeView.Nodes[i].Nodes[j].ContextMenuStrip = contextMenuStrip1;
                                treeView.Nodes[i].Nodes[j].Tag = mapObject.DVMObjects[i].DvmNode.Nodes[j].Tag;
                             }
                             else
                             {
                                 for (int k = 0; k < treeView.Nodes[i].Nodes[j].Nodes.Count; k++)
                                 {
                                     treeView.Nodes[i].Nodes[j].Nodes[k].ContextMenuStrip = contextMenuStrip1;
                                     treeView.Nodes[i].Nodes[j].Nodes[k].Tag = mapObject.DVMObjects[i].DvmNode.Nodes[j].Nodes[k].Tag;
                                 }
                             }
                         }
                         treeView.Nodes[i].Tag = mapObject.DVMObjects[i].DvmNode.Tag;
                     }

                     if (conCnt > 0)
                     {
                         sendAlert.ConCount[i]++;
                         sendAlert.LastMessage[i] = 4;
                     }
                     else if (syncCnt > 0)
                     {
                         sendAlert.SynCount[i]++;
                         sendAlert.LastMessage[i] = 3; 
                     }
                     else if (redCnt > 0)
                     {
                         sendAlert.RedCount[i]++;
                         sendAlert.LastMessage[i] = 2;
                     }
                     else if (yelCnt > 0)
                     {
                         sendAlert.YelCount[i]++;
                         sendAlert.LastMessage[i] = 1;
                     }
                     else
                     {
                         sendAlert.LastMessage[i] = 0;
                         sendAlert.SetOK(i);
                     }

                     dvmThreads[i] = new Thread(new ThreadStart(mapObject.DVMObjects[i].SetDVM));
                     dvmThreads[i].Start();
                 }
             }
             if (update == true)
             {
                 treeView.EndUpdate();
             }
             try
             {
                 sendAlert.SendTheAlert();
             }
             catch (Exception ex)
             {
                 tlStripErrorLabel.Visible = true;
                 tlStripErrorLabel.Text = ex.Message;
             }
        }

        private void CloseProcess(string process)
        {
            System.Diagnostics.Process[] tasks = System.Diagnostics.Process.GetProcesses();
            foreach (System.Diagnostics.Process p in tasks)
            {
                if (p.MainWindowTitle.ToString().Contains(process))
                {
                    p.Kill();
                }
            }
        }

        private bool IsProcess(string process)
        {
            System.Diagnostics.Process[] tasks = System.Diagnostics.Process.GetProcesses();
            foreach (System.Diagnostics.Process p in tasks)
            {
                if (p.MainWindowHandle.ToString().Contains(process))
                {
                    return true;
                }
            }
            return false;
        }

        private void MainFormClosing(object sender, FormClosingEventArgs e)
        {
            if (dvmThreads != null)
            {
                for (int i = 0; i < dvmThreads.Length; i++)
                {
                    if (dvmThreads[i] != null && dvmThreads[i].IsAlive)
                    {
                        dvmThreads[i].Abort();
                    }
                }
            }
            FileIO.FileIO fileIO = new FileIO.FileIO();
            fileIO.WriteRunStatus(run, mapFile);
        }

        private void mailAlertToolStripMenuItem_Click(object sender, EventArgs e)
        {
            MailAlert mailAlert = new MailAlert();
            mailAlert.ShowDialog();
            SetAlertInfo();
        }

        private void mapsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ConfigMap configMap = new ConfigMap();
            configMap.ShowDialog();
            for (int i = 0; i < mapObject.DVMObjects.Length; i++)
            {
                tbPviewMap.Controls.Remove(mapObject.DVMObjects[i].Led);
                tbPviewMap.Controls.Remove(mapObject.DVMObjects[i].MapText);
            }

            Xml_Serializer xmlS = new Xml_Serializer(System.Windows.Forms.Application.StartupPath + "/mailsettings.xml");
            sendAlert = new SendAlert(mapObject.DVMObjects.Length, (AlertSettings)xmlS.DeserializeObject(typeof(AlertSettings)));
            sendAlert.DVMObjects = mapObject.DVMObjects;
            treeView.Nodes.Clear();
            InitDvmObj();
            StartMonitoring();
        }

        private void splitContainer_SplitterMoved(object sender, SplitterEventArgs e)
        {
            if (mapObject != null)
            {
                this.SuspendLayout();
                for (int i = 0; i < mapObject.DVMObjects.Length; i++)
                {
                    mapObject.DVMObjects[i].MapText.Top = Convert.ToInt32(mapObject.DVMObjects[i].Top) + /*splitContainer.Top + */17 + pBMap.Top;
                    mapObject.DVMObjects[i].MapText.Left = Convert.ToInt32(mapObject.DVMObjects[i].Left) + /*splitContainer.Left + splitContainer.SplitterDistance*/ -63 + pBMap.Left;

                    mapObject.DVMObjects[i].Led.Top = Convert.ToInt32(mapObject.DVMObjects[i].Top) + /*splitContainer.Top + */ pBMap.Top;
                    mapObject.DVMObjects[i].Led.Left = Convert.ToInt32(mapObject.DVMObjects[i].Left) + /*splitContainer.Left + splitContainer.SplitterDistance + */ pBMap.Left;
                }
                this.ResumeLayout();
            } 
        }

        private void tlStrpConfAlert_Click(object sender, EventArgs e)
        {
            MailAlert mailAlert = new MailAlert();
            mailAlert.ShowDialog();

            SetAlertInfo();
        }

        private void tlStrpLoadMap_Click(object sender, EventArgs e)
        {
            OFFF.OpenFileFromFolder offf = new OFFF.OpenFileFromFolder();
            offf.Folder = "maps";
            offf.Filter = "XML-Map files|*.xml";
            if (offf.ShowDialog() == DialogResult.OK)
            {
                if (mapObject != null)
                {
                    for (int i = 0; i < mapObject.DVMObjects.Length; i++)
                    {
                        tbPviewMap.Controls.Remove(mapObject.DVMObjects[i].Led);
                        tbPviewMap.Controls.Remove(mapObject.DVMObjects[i].MapText);
                    }
                }
                mapFile = offf.Folder + offf.FileName;
                this.LoadMapFromFile(Application.StartupPath + "/" + mapFile);
            }
        }

        private void tlStrpSaveMap_Click(object sender, EventArgs e)
        {
            bool saved = true;
            if (mapObject != null)
            {
                if (mapObject.StartUpMap == true)
                {
                    try
                    {
                        File.WriteAllText(Application.StartupPath + "/start.conf", mapFile);
                    }
                    catch
                    {
                        MessageBox.Show("This map could not be set as start-up!");
                    }
                }

                mapObject.StartUpMap = false;
                if (mapFile == String.Empty)
                {
                    SFIF.SaveFileInFolder sfif = new SFIF.SaveFileInFolder();
                    sfif.Folder = "maps";
                    sfif.Filter = "XML-Map files|*.xml";
                    if (sfif.ShowDialog() == DialogResult.OK)
                        mapFile = sfif.Folder + sfif.FileName;
                    else
                        saved = false;
                }
                if (saved == true)
                {
                    try
                    {
                        Xml_Serializer xmlS = new Xml_Serializer(mapFile);
                        xmlS.SerializeObject((object)mapObject);
                    }
                    catch
                    {
                        MessageBox.Show("This map could not be saved!");
                    }
                }
            }
            SetTitle();
        }

        private void tlStripNewMap_Click(object sender, EventArgs e)
        {
            ConfigMap configMap = new ConfigMap();
            configMap.MapObject = new MAPObject();
            configMap.MapObject.DVMObjects = new DVMObject[0];
            configMap.MapObject.ImgFile = "";
            if (configMap.ShowDialog() == DialogResult.OK)
            {
                if (mapObject != null)
                {
                    for (int i = 0; i < mapObject.DVMObjects.Length; i++)
                    {
                        tbPviewMap.Controls.Remove(mapObject.DVMObjects[i].Led);
                        tbPviewMap.Controls.Remove(mapObject.DVMObjects[i].MapText);
                    }
                }
                mapObject = configMap.MapObject;
                this.LoadMap(mapObject);
                mapFile = String.Empty;
                SetTitle();
            }
        }

        private void LoadMapFromFile(String mapFile)
        {
            MAPObject tempMap = new MAPObject();
            try
            {
                Xml_Serializer xmlS = new Xml_Serializer(mapFile);
                tempMap = (MAPObject)xmlS.DeserializeObject(typeof(MAPObject));
                this.LoadMap(tempMap);
                SetTitle();
            }
            catch(Exception ex)
            {
                Xml_Serializer xmlS = new Xml_Serializer(System.Windows.Forms.Application.StartupPath + "/mailsettings.xml");
                sendAlert = new SendAlert(0, (AlertSettings)xmlS.DeserializeObject(typeof(AlertSettings)));
                MessageBox.Show("This map could not be opened!" + ex.Message);
            }
        }

        private void SetTitle()
        {
            if(mapFile == null)
                MessageBox.Show("There is no map you can save. Please create a new one or load a map");
            else
            {
                if (mapFile != String.Empty)
                    this.Text = mapFile.Substring(5, mapFile.Length - 5);
                else
                    this.Text = "New Map";
                this.Text += " - SNMP Example: DVM Management Center";
            }
        }

        private void LoadMap(MAPObject tempMap)
        {
            try
            {
                pBMap.Load(Application.StartupPath + "/" + tempMap.ImgFile);
                Xml_Serializer xmlS = new Xml_Serializer(System.Windows.Forms.Application.StartupPath + "/mailsettings.xml");
                sendAlert = new SendAlert(tempMap.DVMObjects.Length, (AlertSettings)xmlS.DeserializeObject(typeof(AlertSettings)));
                sendAlert.DVMObjects = tempMap.DVMObjects;
                this.mapObject = tempMap;
                InitDvmObj();
            }
            catch(Exception ex)
            {
                throw new Exception(ex.Message);
            }
        }

        private void tlStrpConfMap_Click(object sender, EventArgs e)
        {
            if (mapObject != null)
            {
                this.StopMonitoring();
                ConfigMap configMap = new ConfigMap();
                configMap.MapFile = mapFile;
                DVMObject[] tempDVMs = new DVMObject[mapObject.DVMObjects.Length];
                int i = 0;
                foreach (DVMObject tempDVM in mapObject.DVMObjects)
                {
                    tempDVMs[i] = tempDVM;
                    i++;
                }
                configMap.MapObject = mapObject;
                if (configMap.ShowDialog() == DialogResult.OK)
                {
                    for (i = 0; i < tempDVMs.Length; i++)
                    {
                        this.tbPviewMap.Controls.Remove(tempDVMs[i].Led);
                        this.tbPviewMap.Controls.Remove(tempDVMs[i].MapText);
                    }
                    mapObject = configMap.MapObject;
                    this.LoadMap(mapObject);
                }
                this.StartMonitoring();
            }
            else
                MessageBox.Show("No map loaded");
        }

        private void SetAlertInfo()
        {
            try
            {
                Xml_Serializer xmlS = new Xml_Serializer(Application.StartupPath + MAIL_CONFIG_FILE);
                AlertSettings alertSettings = (AlertSettings)xmlS.DeserializeObject(typeof(AlertSettings));
                if (alertSettings.BlConnectivity == true)
                {
                    lbConnValue.Visible = true;
                    lbConUnit.Text = "Consecutive events";
                }
                else
                {
                    lbConnValue.Visible = false;
                    lbConUnit.Text = "not active";
                }
                lbConnValue.Text = alertSettings.TbConnectivity.ToString();

                if (alertSettings.BlSyncLoss == true)
                {
                    lbSyncLossValue.Visible = true;
                    lbSyncUnit.Text = "Consecutive events";
                }
                else
                {
                    lbSyncLossValue.Visible = false;
                    lbSyncUnit.Text = "not active";
                }
                lbSyncLossValue.Text = alertSettings.TbSyncLoss.ToString();

                if (alertSettings.BlAlarm == true)
                {
                    lbAlarmValue.Visible = true;
                    lbAlarmUnit.Text = "Consecutive events";
                }
                else
                {
                    lbAlarmValue.Visible = false;
                    lbAlarmUnit.Text = "not active";
                }
                lbAlarmValue.Text = alertSettings.TbAlarm.ToString();

                if (alertSettings.BlEntry == true)
                {
                    lbEntryUnit.Text = "active";
                }
                else
                {
                    lbEntryUnit.Text = "not active";
                }

                lbEmailValue.Text = alertSettings.HoursDelay.ToString() + ((alertSettings.HoursDelay != 1) ? " hours" : " hour");

                if (!alertSettings.BlConnectivity && !alertSettings.BlSyncLoss && !alertSettings.BlAlarm && !alertSettings.BlEntry)
                {
                    lbNoAlert.Text = "Mail alert deactivated";
                    lbNoAlert.ForeColor = Color.Red;
                }
                else
                {
                    lbNoAlert.Text = "Mail alert activated";
                    lbNoAlert.ForeColor = Color.Green;
                }

                lbRecipient.Text = alertSettings.Receiver;
                lbOriginator.Text = alertSettings.Sender;

                if (mapObject != null)
                {
                    sendAlert = new SendAlert(mapObject.DVMObjects.Length, alertSettings);
                    sendAlert.DVMObjects = mapObject.DVMObjects;
                }
                else
                {
                    sendAlert = new SendAlert(0, alertSettings);
                    sendAlert.DVMObjects = new DVMObject[0];
                }
            }
            catch
            {
                MessageBox.Show("The mail settings could not be loaded!");
            }
        }

        private void aboutToolStripMenuItem_Click(object sender, EventArgs e)
        {
            About about = new About();
            about.ShowDialog();
        }

        private void tlStrpPlay_Click(object sender, EventArgs e)
        {
            this.StartMonitoring();
        }

        private void tlStrpStop_Click(object sender, EventArgs e)
        {
            this.StopMonitoring();
        }

        private void StartMonitoring()
        {
            if (mapObject != null)
            {
                if (mapObject.DVMObjects.Length > 0)
                {
                    run = true;
                    tlStrpPlay.Enabled = false;
                    startMonitoringToolStripMenuItem.Enabled = false;
                    stopMonitoringToolStripMenuItem.Enabled = true;
                    tlStrpStop.Enabled = true;
                    timer_check.Start();
                    tlStrpStLb.BackColor = Color.Lime;
                    tlStrpStLb.Text = "running";
                }
                else
                    MessageBox.Show("No DVMs are part of the current map.");
            }
            else
                MessageBox.Show("There is no map loaded.");
        }

        private void StopMonitoring()
        {
            run = false;
            tlStrpPlay.Enabled = true;
            startMonitoringToolStripMenuItem.Enabled = true;
            stopMonitoringToolStripMenuItem.Enabled = false;
            tlStrpStop.Enabled = false;
            timer_check.Stop();
            timer_progress.Enabled = false;
            tlStrpStPr.Value = 0;

            for (int i = 0; i < treeView.Nodes.Count; i++)
            {
                mapObject.DVMObjects[i].Led.BackColor = Color.LightGray;
                mapObject.DVMObjects[i].Led.Text = "";
                treeView.Nodes[i].SelectedImageIndex = treeView.Nodes[i].ImageIndex = IconImagelist.MAP_NO_MONITORING;
                treeView.Nodes[i].Tag = "no monitoring";
            }
            tlStrpStPr.Value = 0;
            tlStrpStLb.BackColor = Color.Red;
            tlStrpStLb.Text = "stopped";
        }

        private void timer_progress_Tick(object sender, EventArgs e)
        {
            if (tlStrpStPr.Value >= 100)
                timer_progress.Enabled = false;
            else
                tlStrpStPr.Value += 10;
        }

        private void tabControl1_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (tabControl1.SelectedIndex == 1)
                timer_logs.Start();
            else
                timer_logs.Stop();
        }

        private void timer_logs_Tick(object sender, EventArgs e)
        {
            if (sendAlert != null)
            {
                if (sendAlert.TempLogs.Count > 0)
                {
                    lsVlogs.BeginUpdate();
                    for (int i = 0; i < sendAlert.TempLogs.Count; i++)
                    {
                        lsVlogs.Items.Insert(0, sendAlert.TempLogs[i].ToString());
                    }
                    sendAlert.TempLogs = new Hashtable();
                    lsVlogs.EndUpdate();
                }
            }
        }

        private void timer_lastlog_Tick(object sender, EventArgs e)
        {
            if (sendAlert != null)
            {
                if (sendAlert.LastLog.Length >= 1)
                {
                    tlStrpStStatus.Text = sendAlert.LastLog.Substring(1);
                    int status = Int32.Parse(sendAlert.LastLog.Substring(0, 1));
                    switch (status)
                    {
                        case 0:
                            tlStrpStStatus.Image = imageList.Images[IconImagelist.NO_ALARM];
                            tlStrpStStatus.Text = "No entries in the log file";
                            break;
                        case 1: // history alarm
                            tlStrpStStatus.Image = imageList.Images[IconImagelist.HISTORY_ALARM];
                            break;
                        case 2: //alarm
                            tlStrpStStatus.Image = imageList.Images[IconImagelist.CURRENT_ALARM];
                            break;
                        case 3: //syncloss
                            tlStrpStStatus.Image = imageList.Images[IconImagelist.SYNC_LOSS];
                            break;
                        case 4: //connecitivity loss
                            tlStrpStStatus.Image = imageList.Images[IconImagelist.MAP_CONNECTIVITY_LOSS];
                            break;
                    }
                }
            }
        }

        private void tlStrpStStatus_Click(object sender, EventArgs e)
        {
            tabControl1.SelectedIndex = 1;
        }

        private void button1_Click(object sender, EventArgs e)
        {
            if (mapObject != null)
            {
                if (mapObject.DVMObjects.Length > 0)
                {
                    sendAlert.SentTime = new DateTime[mapObject.DVMObjects.Length];
                    MessageBox.Show("Email delay was set back.");
                }
                else
                    MessageBox.Show("No DVMs are part of the current map.");
            }
            else
                MessageBox.Show("There is no map loaded.");
        }

        private void treeView_AfterExpand(object sender, TreeViewEventArgs e)
        {
            if (treeView.SelectedNode != null && treeView.SelectedNode.Tag != null)
            {
                int index;
                Int32.TryParse(treeView.SelectedNode.Tag.ToString(), out index);
                if (index != 0)
                {
                    if (treeView.SelectedNode.ImageIndex == index)
                        treeView.SelectedNode.ImageIndex = treeView.SelectedNode.SelectedImageIndex = IconImagelist.OPENED_FOLDER; // expanded folder
                }
            }
        }

        private void treeView_AfterCollapse(object sender, TreeViewEventArgs e)
        {
            if (treeView.SelectedNode != null)
            {
                if (treeView.SelectedNode.ImageIndex == IconImagelist.OPENED_FOLDER)
                    treeView.SelectedNode.ImageIndex = treeView.SelectedNode.SelectedImageIndex = treeView.SelectedNode.ImageIndex = Int32.Parse(treeView.SelectedNode.Tag.ToString()); // collapsed folder
            }
        }

        private void toolStripMenuItem1_Click(object sender, EventArgs e)
        {
            try
            {
                String[] infos = treeView.SelectedNode.Tag.ToString().Split('|');
                DVMAppFuncs dvm = new DVMAppFuncs(infos[0], infos[1], infos[2], Application.StartupPath);
                dvm.ClearLogs(Int32.Parse(infos[3]), Int32.Parse(infos[4]));
            }
            catch { }
        }

        private void btClearLogs_Click(object sender, EventArgs e)
        {
            FileIO.FileIO file = new FileIO.FileIO();
            file.ClearFile(Application.StartupPath + "/" + CURRENT_LOG_FILE);
            lsVlogs.Items.Clear();
        }

        private void tlStripErrorLabel_Click(object sender, EventArgs e)
        {
            tlStripErrorLabel.Visible = false;
        }
    }
}