/*Class name: SendMail
 *Namespace: DVM_Management_Center.Mail
 *Author: Benno Rott
 *Last change: Benno Rott
 *Changed: 21/02/2007
 *Use: Provides basic routines for sending an email
 */
using System;
using System.Collections;
using System.Net.Mail;
using System.Net;

namespace DVM_Management_Center.Mail
{
	/// <summary>
	/// Class for sending an email to a given email-adress with smtp
	/// </summary>
	public class SendMail
	{
        SmtpClient smtpClient;
        /// <summary>
        /// This class includes methods for sending an email
        /// </summary>
        /// <param name="host">The host of the smtp-server</param>
        /// <param name="port">The port of the smtp-server</param>
        /// <param name="username">The optional username for the smtp-server</param>
        /// <param name="password">The optional password for the smtp-server</param>
		public SendMail(String host, String port, String username, String password)
		{
                int iPort = 0;
                Int32.TryParse(port, out iPort);
                smtpClient = new SmtpClient(host, iPort);
                smtpClient.UseDefaultCredentials = false;
                if (username != String.Empty && password != String.Empty)
                {
                    smtpClient.DeliveryMethod = SmtpDeliveryMethod.Network;
                    smtpClient.Credentials = new NetworkCredential(username, password);
                }
		}
	    
        /// <summary>
        /// Methode for sending the email
        /// </summary>
        /// <param name="to">The recipient of the email</param>
        /// <param name="from">The sender of the email</param>
        /// <param name="subject">The subject of the email</param>
        /// <param name="body">The body of the email</param>
		public void SendAlert(String to, String from, String subject, String body)
		{
			try
			{
                smtpClient.Send(from, to, subject, body);
			}
			catch (Exception ex)
			{
                String error = "Alert Error: " + ex.Message.ToString().Replace("\r\n", " ");
                if(ex.InnerException != null)
                    error += " - " + ex.InnerException.Message;
                throw new Exception(error);
			}
		}
	}
}
